@@ -11,6 +11,8 @@ from Crypto.Cipher import AES
 from bs4 import BeautifulSoup
 from mkdocs.plugins import BasePlugin
 from mkdocs.config import config_options
+from urllib.parse import urlsplit
+from urllib.request import urlopen
 
 try:
     from mkdocs.utils import string_types
@@ -53,6 +55,8 @@ class encryptContentPlugin(BasePlugin):
         ('encryption_info_message', config_options.Type(string_types, default=str(SETTINGS['encryption_info_message']))),
         ('password_button_text', config_options.Type(string_types, default=str(SETTINGS['password_button_text']))),
         ('password_button', config_options.Type(bool, default=False)),
+        ('input_class', config_options.Type(string_types, default=None)),
+        ('button_class', config_options.Type(string_types, default=None)),
         # password feature
         ('global_password', config_options.Type(string_types, default=None)),
         ('use_secret', config_options.Type(string_types, default=None)),
@@ -74,10 +78,13 @@ class encryptContentPlugin(BasePlugin):
         ('encrypted_something', config_options.Type(dict, default={})),
         ('search_index', config_options.Choice(('clear', 'dynamically', 'encrypted'), default='encrypted')),
         ('reload_scripts', config_options.Type(list, default=[])),
-        ('experimental', config_options.Type(bool, default=False)),
+        ('inject', config_options.Type(dict, default={})),
+        ('selfhost', config_options.Type(bool, default=False)),
+        ('selfhost_download', config_options.Type(bool, default=True)),
         # legacy features, doesn't exist anymore
         ('disable_cookie_protection', config_options.Type(bool, default=False)),
-        ('decrypt_search', config_options.Type(bool, default=False))
+        ('decrypt_search', config_options.Type(bool, default=False)),
+        ('experimental', config_options.Type(bool, default=False)),
     )
 
     def __hash_md5__(self, text):
@@ -103,9 +110,18 @@ class encryptContentPlugin(BasePlugin):
             PADDING_CHAR
         )
 
-    def __encrypt_content__(self, content, base_path):
+    def __encrypt_content__(self, content, password, base_path, encryptcontent_path):
         """ Replaces page or article content with decrypt form. """
-        ciphertext_bundle = self.__encrypt_text_aes__(content, str(self.config['password']))
+        ciphertext_bundle = self.__encrypt_text_aes__(content, password)
+
+        # optionally selfhost cryptojs
+        if self.config["selfhost"]:
+            js_libraries = []
+            for jsurl in JS_LIBRARIES:
+                js_libraries.append(base_path + 'assets/javascripts/cryptojs/' + jsurl.rsplit('/',1)[1])
+        else:
+            js_libraries = JS_LIBRARIES
+
         decrypt_form = Template(self.config['html_template']).render({
             # custom message and template rendering
             'summary': self.config['summary'],
@@ -113,11 +129,14 @@ class encryptContentPlugin(BasePlugin):
             'password_button': self.config['password_button'],
             'password_button_text': self.config['password_button_text'],
             'encryption_info_message': self.config['encryption_info_message'],
+            'input_class': self.config['input_class'],
+            'button_class': self.config['button_class'],
             # this benign decoding is necessary before writing to the template,
             # otherwise the output string will be wrapped with b''
             'ciphertext_bundle': b';'.join(ciphertext_bundle).decode('ascii'),
-            'js_libraries': JS_LIBRARIES,
+            'js_libraries': js_libraries,
             'base_path': base_path,
+            'encryptcontent_path': encryptcontent_path,
             # add extra vars
             'extra': self.config['html_extra_vars']
         })
@@ -138,7 +157,8 @@ class encryptContentPlugin(BasePlugin):
             'default_expire_delay': int(self.config['default_expire_delay']),
             'encrypted_something': self.config['encrypted_something'],
             'reload_scripts': self.config['reload_scripts'],
-            'experimental': self.config['experimental'],
+            'experimental': self.config['search_index'] == 'dynamically',
+            'site_path': self.config['site_path'],
             # add extra vars
             'extra': self.config['js_extra_vars']
         })
@@ -212,12 +232,17 @@ class encryptContentPlugin(BasePlugin):
         # Re order plugins to be sure search-index are not encrypted
         if self.config['search_index'] == 'clear':
             logger.debug('Reordering plugins loading and put search and encryptcontent at the end of the event pipe.')
-            config['plugins'].move_to_end('search')
-            config['plugins'].move_to_end('encryptcontent')
+            try: #PluginCollection is no instance of OrderedDict anymore since MkDocs 1.4
+                config['plugins'].move_to_end('search')
+                config['plugins'].move_to_end('encryptcontent')
+            except:
+                logger.warning('Could not reorder plugins. Please check that "search" and "encryptcontent" are at the end of plugins')
         # Enable experimental code .. :popcorn:
-        if self.config['search_index'] == 'dynamically':
-            logger.info('EXPERIMENTAL MODE ENABLE. Only work with default SearchPlugin, not Material.')
-            self.config['experimental'] = True
+        elif self.config['search_index'] == 'dynamically':
+            logger.info("EXPERIMENTAL search index encryption enabled.")
+        self.config['encrypted_something'] = self.config['inject'] | self.config['encrypted_something'] #add inject to encrypted_something
+        # Get path to site in case of subdir in site_url
+        self.config['site_path'] = urlsplit(config.data["site_url"] or '/').path[1::]
 
     def on_pre_build(self, config, **kwargs):
         """
@@ -232,50 +257,20 @@ class encryptContentPlugin(BasePlugin):
         # encrypt 'text' fields of search_index.
         # ref: https://github.com/mkdocs/mkdocs/tree/master/mkdocs/contrib/search
         try:
-            if self.config['search_index'] in ['encrypted', 'dynamically']:
-                from mkdocs.contrib.search.search_index import SearchIndex, ContentParser
-                def _create_entry_for_section(self, section, toc, abs_url, password=None):
-                    toc_item, text = self._find_toc_by_id(toc, section.id), ''
-                    if not self.config.get('indexing') or self.config['indexing'] == 'full':
-                        text = ' '.join(section.text)
-                    if password is not None:
-                        plugin = config['plugins']['encryptcontent']
-                        code = plugin.__encrypt_text_aes__(text, str(password))
-                        text = b';'.join(code).decode('ascii')
-                    if toc_item is not None:
-                        self._add_entry(title=toc_item.title, text=text, loc=abs_url + toc_item.url)
-                SearchIndex.create_entry_for_section = _create_entry_for_section
-                def _add_entry_from_context(self, page):
-                    parser, url, text = ContentParser(), page.url, ''
-                    parser.feed(page.content)
-                    parser.close()
-                    if not self.config.get('indexing') or self.config['indexing'] == 'full':
-                        text = parser.stripped_html.rstrip('\n')
-                    if (hasattr(page, 'encrypted') and hasattr(page, 'password') and page.password is not None):
-                        plugin = config['plugins']['encryptcontent']
-                        code = plugin.__encrypt_text_aes__(text, str(page.password))
-                        text = b';'.join(code).decode('ascii')
-                    self._add_entry(title=page.title, text=text, loc=url)
-                    if (self.config.get('indexing') and self.config['indexing'] in ['full', 'sections']):
-                        for section in parser.data:
-                            if (hasattr(page, 'encrypted') and hasattr(page, 'password') and page.password is not None):
-                                self.create_entry_for_section(section, page.toc, url, page.password)
-                            else:
-                                self.create_entry_for_section(section, page.toc, url)
-                SearchIndex.add_entry_from_context = _add_entry_from_context
-            if self.config['experimental'] is True:
+            #search_index encryption was moved to on_post_page
+            if self.config['search_index'] == 'dynamically':
                 if config['theme'].name == 'material':
-                    logger.error("UNSUPPORTED Material theme with experimantal feature search_index=dynamically !")
-                    exit("UNSUPPORTED Material theme: use search_index: [clear|encrypted] instead.")
-                # Overwrite search/*.js files from templates/search with encryptcontent contrib search assets
-                config['theme'].dirs = [
-                    e for e in config['theme'].dirs
-                    if not re.compile(r".*/contrib/search/templates$").match(e)
-                ]
-                path = os.path.join(base_path, 'contrib/templates')
-                config['theme'].dirs.append(path)
-                if 'search/main.js' not in config['extra_javascript']:
-                    config['extra_javascript'].append('search/main.js')
+                    logger.warning("To enable EXPERIMENTAL search index decryption mkdocs-material needs to be customized (patched)!")
+                else:
+                    # Overwrite search/*.js files from templates/search with encryptcontent contrib search assets
+                    for dir in config['theme'].dirs.copy():
+                        if re.compile(r".*[/\\]contrib[/\\]search[/\\]templates$").match(dir):
+                            config['theme'].dirs.remove(dir)
+                            path = os.path.join(base_path, 'contrib/templates')
+                            config['theme'].dirs.append(path)
+                            if 'search/main.js' not in config['extra_javascript']:
+                                config['extra_javascript'].append('search/main.js')
+                            break
         except Exception as exp:
             logger.exception(exp)
 
@@ -320,12 +315,11 @@ class encryptContentPlugin(BasePlugin):
             if self.config['tag_encrypted_page']:
                 # Set attribute on page to identify encrypted page on template rendering
                 setattr(page, 'encrypted', True)
-            # Create relative path in case of subdir in site_url
-            base_path = page.abs_url.replace(page.url, '') if page.abs_url is not None else '/'
             # Set password attributes on page for other mkdocs events
             setattr(page, 'password', str(self.config['password']))
-            # Keep encrypted html as temporary variable on page cause we need clear html for search plugin
-            setattr(page, 'html_encrypted', self.__encrypt_content__(html, base_path))
+            # Keep encrypted html to encrypt as temporary variable on page
+            if not self.config['inject']:
+                setattr(page, 'html_to_encrypt', html)
         return html
 
     def on_page_context(self, context, page, config, **kwargs):
@@ -341,9 +335,21 @@ class encryptContentPlugin(BasePlugin):
         :param nav: global navigation object
         :return: dict of template context variables
         """
-        if hasattr(page, 'html_encrypted'):
-            page.content = page.html_encrypted
-            delattr(page, 'html_encrypted')
+        if hasattr(page, 'html_to_encrypt'):
+            page.content = self.__encrypt_content__(
+                page.html_to_encrypt, 
+                str(page.password),
+                context['base_url']+'/',
+                self.config['site_path']+page.url
+            )
+            delattr(page, 'html_to_encrypt')
+        if self.config['inject']:
+            setattr(page, 'decrypt_form', self.__encrypt_content__(
+                '<!-- dummy -->', 
+                str(page.password),
+                context['base_url']+'/',
+                self.config['site_path']+page.url
+            ))
         return context
 
     def on_post_page(self, output_content, page, config, **kwargs):
@@ -389,10 +395,45 @@ class encryptContentPlugin(BasePlugin):
                             if isinstance(item['style'], list):
                                 item['style'].append("display:none")
                             else:
-                                item['style'] = item['class'] + "display:none"
+                                # if style contains a single element (str)
+                                item['style'] = item['style'] + "display:none"
                         else:
                             item['style'] = "display:none"
+            if self.config['inject'] and len(self.config['inject']) == 1:
+                name, tag = list(self.config['inject'].items())[0]
+                injector = soup.new_tag("div")
+                something_search = soup.find(tag[0], {tag[1]: name})
+                something_search.insert_before(injector)
+                injector.append(BeautifulSoup(page.decrypt_form, 'html.parser'))
+                delattr(page, 'decrypt_form')
             output_content = str(soup)
+
+        if hasattr(page, 'encrypted'):
+            #encrypt or exclude encrypted pages from search_index.json
+            for plugin in config['plugins']:
+                if plugin.endswith('search'):
+                    try:
+                        search_entries = config['plugins'][plugin].search_index._entries
+                        for entry in search_entries.copy(): #iterate through all entries of search_index
+                            location = page.url.lstrip('/')
+                            if entry['location'] == location or entry['location'].startswith(location+"#"): #find the ones located at encrypted pages
+                                if self.config['search_index'] == 'encrypted':
+                                    search_entries.remove(entry)
+                                elif self.config['search_index'] == 'dynamically' and page.password is not None:
+                                    #encrypt text/title/location(anchor only)
+                                    text = entry['text']
+                                    title = entry['title']
+                                    toc_anchor = entry['location'].replace(location, '')
+                                    code = self.__encrypt_text_aes__(text, page.password)
+                                    entry['text'] = b';'.join(code).decode('ascii')
+                                    code = self.__encrypt_text_aes__(title, page.password)
+                                    entry['title'] = b';'.join(code).decode('ascii')
+                                    code = self.__encrypt_text_aes__(toc_anchor, page.password)
+                                    entry['location'] = location + ';' + b';'.join(code).decode('ascii')
+                        config['plugins'][plugin].search_index._entries = search_entries
+                    except:
+                        logger.error('Could not encrypt search index of "' + page.title + '" for "' + plugin+ '" plugin!')
+
         return output_content
 
     def on_post_build(self, config, **kwargs):
@@ -406,3 +447,14 @@ class encryptContentPlugin(BasePlugin):
         decrypt_js_path = Path(config.data["site_dir"] + '/assets/javascripts/decrypt-contents.js')
         with open(decrypt_js_path, "w") as file:
             file.write(self.__generate_decrypt_js__())
+
+        # optionally download cryptojs
+        if self.config["selfhost"] and self.config["selfhost_download"]:
+            logger.info('Downloading cryptojs for self-hosting... Please consider copying "assets/javascripts/cryptojs/" to "doc/" and setting "selfhost_download: false" to decrease build time.')
+            Path(config.data["site_dir"] + '/assets/javascripts/cryptojs/').mkdir(parents=True, exist_ok=True)
+            for jsurl in JS_LIBRARIES:
+                dlurl = "https:" + jsurl
+                filepath = Path(config.data["site_dir"] + '/assets/javascripts/cryptojs/' + jsurl.rsplit('/',1)[1])
+                with urlopen(dlurl) as response:
+                    with open(filepath, 'wb') as file:
+                        file.write(response.read())
\ No newline at end of file
